/**
 * Name    : Edison Chindrawaly
 * Class   : CSCI 4450 Spring 2002
 * Compile : g++ -o anagram anagram.cpp
 * Files   : anagram.cpp & words.txt
 * Usage   : ./anagram 
 *           ./anagram [source of dictionary]
 *           ./anagram [source of dictionary] [output file]
 *           ./anagram [source of dictionary] [output file] [query index]
 * Descript: Write a program that takes in some text and output all
 *           the 2-word anagrams (using words drawn from a word list)
 */

#include <iostream.h>
#include <fstream.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <sys/time.h>

#define MAX_LINE 30
#define MAX_CAP 200000
#define NOT_FOUND -1
#define DEF_FILE "words.txt"
#define DEF_OUT "output.txt"

/**********global variables***************************/
char dict[MAX_CAP+1][MAX_LINE+1];
int original_len;
/*****************************************************/

/****************prototype****************************/
void getWord(int,int,char[],char[]);
void permute(int,int,char[],ofstream&);
void process(int, char[],ofstream&);
void swap(int,int,char[]);
void converse(char[]);
int binSearch(char[],int,int);
int readFile(char[]);
void copyArray(int, char[]);
void displayArray(int);
double times();
/****************************************************/

void main(int argc, char** argv)
{
 int index;
 double z;
 char buffer[MAX_LINE+1];
 char filename[MAX_LINE+1];
 char outfile[MAX_LINE+1];
 buffer[0] = '\0';
  
 switch(argc)
 {
  case 1: strcpy(filename,DEF_FILE); 
          strcpy(outfile,DEF_OUT);
          index =0; break;
  case 2: strcpy(filename,argv[1]); 
          strcpy(outfile,DEF_OUT);
          index = 0; break;
  case 3: strcpy(filename,argv[1]); 
          strcpy(outfile,argv[2]);
          index = 0; break;
  case 4: strcpy(filename,argv[1]);
          strcpy(outfile,argv[2]);
          index = atoi(argv[3]); break;
  default: cout<<"You may only put up to 3 arguments"<<endl;
           cout<<"Usage:"<<endl;
           cout<<"./anagram "<<endl;
           cout<<"./anagram dictionary "<<endl;
           cout<<"./anagram dictionary outfile "<<endl;
           cout<<"./anagram dictionary outfile queryIndex"<<endl;
           cout<<"for example: "<<endl;
           cout<<"./anagram dictionary.txt output.txt 10000"<<endl;
           exit(1);
 }

 int count = readFile(filename);
 cout<<"Load "<<count<<" words from "<<filename<<endl;
 cout<<"> ";
 cin>>buffer;
 converse(buffer);
 int got = binSearch(buffer,0,count);
 if(got == NOT_FOUND)
 {
  cout<<buffer<<" is not in the dictionary."<<endl;
  exit(0);
 }

 cout<<"Found at index "<<got<<" : ";
 displayArray(got);

 ofstream out (outfile);
 if(!out.is_open())
 {
  cout<<"Error in opening "<<outfile<<endl;
  exit(1);
 }

 int len = strlen(buffer);
 original_len = len;
 z = times();
 for(int i=len; i>0; i--)
   permute(i,count,buffer,out);
 double diffTime = times() - z; 
 out.close();
 printf("%s %.5f %s\n","Process takes ", diffTime, " seconds.");

 if(index > count)
 {
  cout<<"your query is bigger than the available words"<<endl;
  exit(1);
 }

 if(index != 0)
 {
  cout<<index<<" word: ";
  copyArray(index-1,buffer);
  cout<<buffer<<" > length = "<<strlen(buffer)<<endl;
 } 
}

/**
 * converse is to check for non-alphanumeric character
 * and to convert everything to lower case
 * @param  char[] source
 * @return none
 */
void converse(char source[])     
{
 char tmp[MAX_LINE+1];
 tmp[0]='\0';
 int len = strlen(source);
 int j=0;
 for(int i=0; i<len; i++)
 {
  if(isalpha(source[i]))
  {
    source[i]=tolower(source[i]);
    tmp[j++] = source[i]; 
  } 
 }
 tmp[j] = '\0';
 strcpy(source,tmp);
}

/**
 * copyArray is to copy the word at the dictionary
 * specified by the index into the dest[]
 * @param int index , char dest[]
 * @return none
 */
void copyArray(int index, char dest[])
{
 int i;
 for(i=0; i<MAX_LINE; i++)
  if(dict[index][i] != '\0')
    dest[i]=dict[index][i];
  else
    break;
 dest[i]='\0';
}

/**
 * displayArray is to display the word in dictionary 
 * specified by index
 * @param int index
 * @return none
 */
void displayArray(int index)
{
 int i;
 for(i=0; i<MAX_LINE; i++)
   if(dict[index][i] != '\0')
     cout<<dict[index][i]; 
   else
   {
     cout<<endl;
     break;
   }
}

/**
 * readFile method is to read from file and input it
 * into the global 2D array called dict or dictionary
 * @param char filename[]
 * @return int number of input
 */
int readFile(char x[])
{
 ifstream f (x);
 if(!f.is_open()) 
 {
  cout<<"Error in opening: "<<x<<endl;
  return 0;
 }
 char tmp[MAX_LINE+1];
 int count=0;
 while(!f.eof())
 {
  f.getline(tmp,MAX_LINE);
  strcpy(dict[count],tmp);
  count++;
 }
 f.close();
 return count-1; 
}

/**
 * binSearch is variant of implementation of binary search
 * @param char x[], int begin, int end
 * @return int NOT_FOUND  (-1) else it returns the index
 */
int binSearch(char x[], int left, int right)
{
 char tmp[MAX_LINE+1];
 if(left == right)
 {
  copyArray(left,tmp);
  if(strcmp(x,tmp) == 0)
    return left; 
  else
    return NOT_FOUND;
 }
 else
 {
  int m = (left+right)/2;
  copyArray(m,tmp);
  if(strcmp(x,tmp)<0)
   return binSearch(x,left,m);
  else if(strcmp(x,tmp)>0)
   return binSearch(x,m+1,right);
  else //found
   return m;
 }
}

double times()
{
 struct timeval tv;
 gettimeofday(&tv,NULL);
 return tv.tv_sec+(float)(tv.tv_usec)/1000000.0;
}

/**
 * permute method is to permutate all the possibility
 * @param int m (the index), int count (the max word in dict),
 *        char A[] (the word to permutate), ofstream (the
 *        tool that writes to output file)
 * @return none
 */
void permute(int m, int count, char A[], ofstream& out)
{
 if(m == 0)
   process(count,A,out);
 else
 {
  permute(m-1,count,A,out);
  for(int i=m-1; i>=0; i--)
  {
   swap(m,i,A);
   permute(m-1,count,A,out);
   swap(m,i,A);
  }
 }
}

/**
 * process method is to process the result of permute.
 * This is where you split the word into anagram and
 * check it against the dictionary.
 * @param int count, char word[], ofstream&
 * @return none
 */
void process(int count,char word[], ofstream& out)
{
 int len = strlen(word);
 char word1[MAX_LINE+1]; word1[0]='\0';
 char word2[MAX_LINE+1]; word2[0]='\0';
 int found1, found2;
 for(int i=len; i>0; i--)
 {
  getWord(0,i,word,word1);
  getWord(i,len,word,word2); 
  int totlen = strlen(word1)+strlen(word2);
  if(totlen < original_len)
    break; 
  found1 = 0; found2 = 0;
  if(strlen(word1) != 0)
    found1 = binSearch(word1,0,count);    
  if(strlen(word2) != 0)
    found2 = binSearch(word2,0,count); 
  if((found1!=NOT_FOUND)&&(found2!=NOT_FOUND))
    out<<word1<<" "<<word2<<endl;
 }
}

/**
 * getWord is to divide word (a[]) into part of word (b[])
 * using the specificiation of pos and length
 * @param int pos, int length, char a[], char b[]
 * @return none
 */
void getWord(int pos, int length, char a[], char b[])
{
 int i,j=0;
 for(i=pos;i<length;i++)
   b[j++]=a[i];
 b[j] = '\0'; 
}

/**
 * swap's method is to swap character at two position
 * @param int m, int i, char word[]
 * @return none
 */
void swap(int m, int i, char word[])
{
 if(strlen(word) < 2)
   return;
 char tmp = word[m];
 word[m] = word[i];
 word[i] = tmp;
}
